"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.APIEventHandler = void 0;
const ajv_1 = __importDefault(require("ajv"));
const path_param_parser_1 = require("./path-param-parser");
const FP = __importStar(require("fp-ts"));
const open_api_1 = require("./open-api");
const logging_1 = require("./logging");
const These_1 = require("fp-ts/lib/These");
const ajv = new ajv_1.default({ strict: false });
const buildCorsHeaders = (req, cfg) => ({
    "Access-Control-Allow-Headers": cfg.allowHeaders === "*" ? "*" : cfg.allowHeaders.join(", "),
    "Access-Control-Allow-Origin": Array.isArray(cfg.allowOrigin)
        ? cfg.allowOrigin.indexOf(req && req["origin"]) > -1
            ? req["origin"]
            : "null"
        : cfg.allowOrigin,
    "Access-Control-Allow-Methods": cfg.allowMethods === "*" ? "*" : cfg.allowMethods.join(", "),
    "Access-Control-Allow-Credentials": cfg.allowCredentials.toString(),
});
const fromBase64 = (data) => {
    const buff = Buffer.from(data, "base64");
    return buff.toString("utf-8");
};
const APIEventHandler = ({ handlers }, cfg) => {
    const handler = (event, ctx) => {
        const { logger, corsConfig } = cfg || {};
        logging_1.logRequestResponse(event, cfg);
        const thisCorsConfig = corsConfig === true
            ? {
                allowCredentials: true,
                allowHeaders: "*",
                allowOrigin: "*",
                allowMethods: "*",
            }
            : corsConfig;
        const route = handlers
            .filter((h) => event.httpMethod.toLowerCase() === h.method.toLowerCase())
            .filter((h) => {
            const handlerSegments = h.url.split("?")[0].split("/");
            const routeSegments = event.path.split("?")[0].split("/");
            return (handlerSegments.length === routeSegments.length &&
                handlerSegments.every((hs, ix) => hs === routeSegments[ix] || path_param_parser_1.PathParamRegex.test(hs)));
        })[0];
        const corsHeaders = thisCorsConfig
            ? buildCorsHeaders(event.headers, thisCorsConfig)
            : {};
        if (route) {
            const path = route.url.split("?")[0];
            const query = route.url.split("?")[1];
            const pathParams = path_param_parser_1.parsePathParams(decodeURIComponent(event.path), path);
            const queryParams = event.queryStringParameters || event.multiValueQueryStringParameters
                ? path_param_parser_1.parseQueryParams(event.queryStringParameters || {}, event.multiValueQueryStringParameters || {}, query)
                : FP.either.right({});
            const bodyObj = event.body ? JSON.parse(event.isBase64Encoded ? fromBase64(event.body) : event.body) : null;
            const isValidBody = route.body ? ajv.validate(route.body, bodyObj) : true;
            if (isValidBody) {
                const tupled = FP.function.pipe(pathParams, FP.either.chain((p) => FP.function.pipe(queryParams, FP.either.map((qp) => [p, qp]))));
                if (FP.either.isRight(tupled)) {
                    return route
                        .handler({
                        pathParams: tupled.right[0],
                        queryParams: tupled.right[1],
                        body: bodyObj,
                        response: (sc, bod, h) => Promise.resolve({
                            statusCode: sc,
                            body: bod,
                            headers: h,
                        }),
                    }, {
                        originalEvent: event,
                        context: ctx,
                    })
                        .then((r) => ({
                        statusCode: r.statusCode,
                        body: r.body ? JSON.stringify(r.body) : "",
                        headers: {
                            ...corsHeaders,
                            ...{ "content-type": "application/json" },
                            ...cfg === null || cfg === void 0 ? void 0 : cfg.defaultHeaders,
                            ...r.headers,
                        },
                    }))
                        .then((r) => {
                        logging_1.logRequestResponse(r);
                        return r;
                    });
                }
                else {
                    logger &&
                        logger.info(`Unresolvable route`, {
                            body: event.body,
                            path: event.path,
                            query: event.queryStringParameters,
                            headers: event.headers,
                        });
                    return Promise.resolve({
                        statusCode: These_1.isLeft(pathParams) || These_1.isLeft(queryParams) ? 400 : 404,
                        body: JSON.stringify({
                            message: These_1.isLeft(pathParams) || These_1.isLeft(queryParams)
                                ? "Bad Request"
                                : "Not Found",
                        }),
                        headers: corsHeaders,
                    }).then((r) => {
                        logging_1.logRequestResponse(r);
                        return r;
                    });
                }
            }
            else {
                logger &&
                    logger.info(`Request body does not much expected schema`, {
                        body: event.body,
                        path: event.path,
                        query: event.queryStringParameters,
                        headers: event.headers,
                    });
                return Promise.resolve({
                    statusCode: 400,
                    body: JSON.stringify({ message: "Bad request" }),
                    headers: corsHeaders,
                }).then((r) => {
                    logging_1.logRequestResponse(r);
                    return r;
                });
            }
        }
        else {
            logger &&
                logger.info(`Unresolvable route`, {
                    body: event.body,
                    path: event.path,
                    query: event.queryStringParameters,
                    headers: event.headers,
                });
            return Promise.resolve({
                statusCode: 404,
                body: JSON.stringify({ message: "Not found" }),
                headers: corsHeaders,
            }).then((r) => {
                logging_1.logRequestResponse(r);
                return r;
            });
        }
    };
    handler.toOpenApi = (apiInfo, functionArn, iamRoleArn, securitySchemes) => open_api_1.toOpenApi(handlers, "1.0", apiInfo, functionArn, iamRoleArn, securitySchemes);
    return handler;
};
exports.APIEventHandler = APIEventHandler;
